/* 4/2/2021
 * BK11-1K / Bl20-2k Spot Welder Arduino Port by Luca Forattini
 * Use an Arduino Pro Mini 3.3V version!
 * Uses the Ucglib by Oliver Kraus https://github.com/olikraus/ucglib
 * Add it using the Library manager (search for ucglib) - a file needs to be modified to correctly drive this display
 * Please modify the INVERT declaration in the Ucglib C file under libraries\Ucglib\src\clib\ucg_dev_tft_128x160_st7735.c
 * lines 59-60 by commenting out line 59 and removing the commenting on 60:
 * //UCG_C10(0x20),
 * UCG_C10(0x21)
 * 
 * MCU Board has 2 seperate 5 pin connectors, the one near the buzzer is NOT connected to anything and just use for structural purposes 
 * Pay attention just to the other one.
 * Starting from pin 1 (marked by a square solder pad):
 * 1 - VCC (after diode from 10000uF cap) which goes to 3.3V regulator on MCU board
 * 2 - N.C.
 * 3 - Input SENSE contact for AUTO Trigger
 * 4 - OUTPUT TO FET Transistor Driver Base
 * 5 - GND
 * 
 * You'll need patience and some soldering skills to wire the Arduino up based on the below indicated inputs. You'll need 13 wires,
 * 11 are signal wires 5 of which for the OLED SPI connection and 2 are power. You can wire the power input of the arduino using 
 * the output coming directly from sfter the switch to the "RAW" (not VCC) input Pin as the Arduino has also his own 3.3V regulator. 
 * It will help with filtering and protecting it from transients (input is also filtered through cap). BE SURE TO WIRE THE GROUND CONNECTION
 * BETWEEN ARDUINO AND MCU BOARD WELL! Failure to do so will result in blowing FETs and/or the Arduino.
 * The hardest ones to wire are the OLED Display pins so take care. You can scrape off some solder mask from tracks to make
 * life easier and solder on there where you have more space. Double check wiring connections with a multimeter.
 * Counting from the outer side of the board the pinout of the ST7735 display is:
 * 
 * OLED           ARDUINO PINS
 *  1 - LED ANODE
 *  2 - GND
 *  3 - RST   -> TO PIN D8 
 *  4 - DC    -> TO PIN D9
 *  5 - SDA   -> TO PIN D11
 *  6 - SCK   -> TO PIN D13
 *  7 - VDD
 *  8 - CS    -> TO PIN D10
 * 
 *  ONLY WIRE INDICATED PINS leave the others be. CHECK FOR SHORTS!
 * 
 * Wiring the 
 * 
 * 6/2/2021 - switched to micros() instead of millis() which yields much better timing accuracy
 *
  UCGLIB notes:
    ROM Size	Font Mode
    9928		NONE
    10942		TRANSPARENT	(1014 Bytes)
    11650		SOLID  (1712 Bytes)
    12214		TRANSPARENT+SOLID

  Hardware SPI Pins:
    Arduino Uno    sclk=13, data=11
    Arduino Due   sclk=76, data=75
    Arduino Mega  sclk=52, data=51   
*/

#include <SPI.h>
#include "Ucglib.h"

#define BT1   7   // First button on the left
#define BT2   6   // Middle button
#define BT3   5   // Right button (Auto / Manual)
#define MTRIG 4   // Manual Trigger Input from jack
#define BUZ   3   // To PIEZO Buzzer connection
#define TRG  2    // To FET Driver Transistor on Pin 4 (wire before 1K or rather 470Ohm resistor coming from PIN 19 of original MCU)
#define SENSE A2  // To Pin 3 on BK board for auto trigger sense
#define BVOLTS A3 // Battery Voltage input
#define TRIGTRIP 100  // Trigger trip value
#define DEFTIME 5
#define TIMEOUT 30000
#define XTPOS 12  // X and Y Timing display position
#define YTPOS 35
#define XVPOS 70  // Battery Voltage Position
#define YVPOS 44
#define XSPOS 85  // Action Text Position
#define YSPOS 65


unsigned long timing = DEFTIME;   // milliseconds
float BVolts = 0;       // Battery Voltage
//int trigsense = 0;    // Auto trigger sense (triggers above 1V through divider)
int T = 0;              // Auto Trigger sense value (triggers above 1V through divider)
int updd = 0;           // display update flag
bool mode = false;      // Auto=1 / Manual=0
bool enabled = false;   // Spot Welder enable safety feature
unsigned long previousmillis = 0;
//unsigned long timeout = 30000;
unsigned long Vsensemillis = 0;
unsigned long long Timermicros = 0;



Ucglib_ST7735_18x128x160_HWSPI ucg(/*cd=*/ 9, /*cs=*/ 10, /*reset=*/ 8);

void setup(void)
{
  digitalWrite(TRG, HIGH);         // Set GATE output LOW (inverted)
  pinMode(TRG, OUTPUT);
  pinMode(MTRIG, INPUT);          // Manual Trigger input
  digitalWrite(MTRIG, HIGH);      // Enable pullups
  pinMode(BT1, INPUT);
  digitalWrite(BT1, HIGH);
  pinMode(BT2, INPUT);
  digitalWrite(BT2, HIGH);
  pinMode(BT3, INPUT);
  digitalWrite(BT3, HIGH);
  pinMode(BUZ, INPUT);
  digitalWrite(BUZ, LOW);
  //delay(500);
  //ucg.begin(UCG_FONT_MODE_TRANSPARENT);
  ucg.begin(UCG_FONT_MODE_SOLID);
  ucg.clearScreen();
  
  //ucg.setColor(255, 255, 255);
  //ucg.drawBox(0, 0, 190, 110);
  //ucg.setFont(ucg_font_ncenR10_tr);
  //ucg.setFont(ucg_font_amstrad_cpc_8r);
  ucg.setFont(ucg_font_8x13_tr);
  ucg.setRotate90();
  ucg.setColor(255, 255, 255);      // Blue Green Red
  //ucg.setColor(1, 0, 0, 255);
  ucg.setPrintPos(1,31);
  //ucg.setColor(1, 64, 64, 255);
  ucg.print("BK11-1K Spot Welder");       // Header Text
  //ucg.setPrintPos(10,95);
  //ucg.setColor(0, 255, 255, 255);       // Font Color (0) B G R
  //ucg.setColor(1, 0, 0, 0);             // background color (1) Blue Green Red
  ucg.drawTriangle(2, 80, 7,96, 12,80);     // Draw up and down arrows for buttons
  ucg.drawTriangle(68, 96, 73,80, 78,96);
  //ucg.print("-");  
  //ucg.setPrintPos(65,95);
  //ucg.print("+");  
  ucg.begin(UCG_FONT_MODE_SOLID);
  ucg.setFont(ucg_font_amstrad_cpc_8r);
  ucg.setRotate90();
  ucg.setPrintPos(XVPOS,YVPOS);
  ucg.print("Batt:     V");           // Battery Voltage Text
  ucg.drawFrame(XTPOS+6, 47, 45, 30); // TIME TEXT FRAME
  ucg.setPrintPos(24,YTPOS+10);       // TIME TEXT POS
  ucg.print("TIME");
  updtiming();
  ucg.setPrintPos(110,95);          // Lower right corner
  //ucg.setColor(0, 0, 0, 0);       // Font Color (0) B G R
  //ucg.setColor(1, 255, 0, 0);     // background color (1) Blue Green Red
  ucg.setColor(1, 0, 0, 0);         // White on Black  B G R
  ucg.setColor(0, 255, 255, 255);   
  ucg.print("  OFF   ");
}

void loop(void)
{
  //ucg.setFont(ucg_font_amstrad_cpc_8r);
  //ucg.setPrintPos(0,0);
  //ucg.setColor(1, 0, 0, 0);
  
  if ((millis()-Vsensemillis) > 2000) {
    Vsensemillis = millis();
    int V = analogRead(BVOLTS);
    ucg.setPrintPos(XVPOS+40,YVPOS);
    ucg.print("     V");
    BVolts = V/51.3;
    if (BVolts<12) {                  // If Voltage below 12V Warn with RED background
      ucg.setColor(1, 0, 0, 255);     // B G R
    } 
    ucg.setPrintPos(XVPOS+40,YVPOS);
    ucg.print(BVolts);  
    ucg.setColor(1, 0, 0, 0);         // White on Black
    //ucg.setPrintPos(XVPOS,YVPOS+10);
    //ucg.print(V);  
  }
  
  if (mode) {                         // Only sense if in AUTO mode
    T = analogRead(SENSE);  
    //ucg.setPrintPos(XSPOS,YSPOS);
    //ucg.print(T);
  }
  
  
  if ((T > TRIGTRIP)&&(mode)&&(enabled)) {
    previousmillis = millis();
    ucg.setPrintPos(XSPOS,YSPOS);
    ucg.print("ARMED!");
    beep(300);
    delay(200);
    ucg.setPrintPos(XSPOS,YSPOS);
    T = analogRead(SENSE);      // Double check if contact is made else cancel
    if (T > TRIGTRIP) {
      ucg.print("FIRED!");
      beep(20);
      pulse();
      beep(50);
      do {
        T = analogRead(SENSE);    // Stay here until contacts released
          } while (T>TRIGTRIP);
    } else {
      ucg.print("CANCEL!");
      delay(100);
    }
    
    //ucg.print(T);
    updd =0;
  } else if (!digitalRead(MTRIG)&&!mode&&enabled) {     // If Manual mode and manual trigger is activated
      previousmillis = millis();
      ucg.setPrintPos(XSPOS,YSPOS);
      ucg.print("FIRING");
      beep(50);
      pulse();
      ucg.setPrintPos(XSPOS,YSPOS);
      ucg.print("FIRED!");
      delay(200);
      beep(100);
      updd =0;
      do {
        
      } while (!digitalRead(MTRIG));
  } else if (!updd) {                 // Update only if necessary
    ucg.setPrintPos(XSPOS,YSPOS);
    ucg.print("       ");
    updd =1;
  }
  
  if (!digitalRead(BT1)){         // decrease Time
    if (timing>1){
      timing--;
      updtiming();  
    }
  } 
  if (!digitalRead(BT2)){         // increase Time
    if (timing<99){
      timing++;
      updtiming();  
    }
  } 
  if (!(digitalRead(BT1)||(digitalRead(BT2)))){
    timing = DEFTIME;
    updtiming();
    beep(200);
  }
  if (!digitalRead(BT3)){
    mode = !(mode);       // FLIP
    beep(50);
    enabled = true;
    previousmillis = millis();
    //ucg.clearScreen();
    ucg.setPrintPos(110,95);
    ucg.setColor(0, 255, 255, 255);   // Font Color (0) B G R
    ucg.setColor(1, 255, 0, 0);       // background color (1) Blue Green Red
    if (mode) {
      ucg.print(" AUTO ");
    } else {
      ucg.setColor(0, 0, 0, 0);       // Font Color (0) B G R
      ucg.setColor(1, 0, 255, 0);     // background color (1) Blue Green Red
      ucg.print("Manual");
    }
    
    ucg.setColor(1, 0, 0, 0);         // set colors back to normal
    ucg.setColor(0, 255, 255, 255);
  }
  
  if (((millis()-previousmillis) > TIMEOUT) && enabled) {
    previousmillis = millis();
    beep(50);
    delay(50);
    beep(150);
    enabled = false;            // disable welding
    mode = false;               // force back to manual when enabled for safety
    ucg.setPrintPos(110,95);
    ucg.setColor(0, 0, 0, 0);       // Font Color (0) B G R
    ucg.setColor(1, 255, 0, 0);     // background color (1) Blue Green Red
    ucg.print("  OFF   ");
    ucg.setColor(1, 0, 0, 0);         // set colors back to normal
    ucg.setColor(0, 255, 255, 255);
  }
  
}

void pulse(){
  //noInterrupts();             // all ints off
  Timermicros = micros();
  digitalWrite(TRG, LOW);    // FET driver circuit Active LOW
  do {                       // delay loop do nothing
    
  } while  (micros() - Timermicros < (timing*1000));    // using micros() instead of millis() increases accuracy by 900 fold - variation is in the tens of uS range instead of ms range!
  //delay(timing);
  digitalWrite(TRG, HIGH);
  //interrupts();
}

void updtiming(){
    previousmillis = millis();
    if (timing < 26) {
      ucg.setColor(1, 0, 240, 0);  
    } else if ((timing < 50) && (timing > 25)) {
      ucg.setColor(1, 100, 100, 255);  
    } else if ((timing < 100) && (timing > 49)) {
      ucg.setColor(1, 0, 0, 255); 
      ucg.setColor(0, 0, 0, 0); 
    }
    ucg.setScale2x2();
    ucg.setPrintPos(XTPOS,YTPOS);
    ucg.print("  ");
    ucg.setPrintPos(XTPOS,YTPOS);
    ucg.print(timing);
    ucg.undoScale();
    ucg.setColor(0, 255, 255, 255);
    ucg.setColor(1, 0, 0, 0);
    beep(10);
    //delay(50);
}

void beep(int len)
{
   pinMode(BUZ, OUTPUT);
   while (len>0) {
      digitalWrite(BUZ, HIGH);
      delay(1);
      digitalWrite(BUZ, LOW);
      delay(1);
      len--;
   }
   pinMode(BUZ, INPUT);
}
